package Mysql::Statement;

use strict;
use vars qw($OPTIMIZE $VERSION $AUTOLOAD);

$VERSION = substr q$Revision: 1.4 $, 10;
# $Id: Statement.pm,v 1.4 1997/10/15 00:10:44 monty Exp $

$OPTIMIZE = 0; # controls, which optimization we default to

sub numrows    { my $x = shift; $x->{'NUMROWS'} or $x->fetchinternal( 'NUMROWS'
  ) }
sub numfields  { shift->fetchinternal( 'NUMFIELDS' ) }
sub affected_rows { my $x = shift; $x->fetchinternal( 'AFFECTEDROWS') }
sub insert_id  { my $x = shift; $x->fetchinternal( 'INSERTID') }
sub table      { return wantarray ? @{shift->fetchinternal('TABLE'    )}: shift->fetchinternal('TABLE'    )}
sub name       { return wantarray ? @{shift->fetchinternal('NAME'     )}: shift->fetchinternal('NAME'     )}
sub type       { return wantarray ? @{shift->fetchinternal('TYPE'     )}: shift->fetchinternal('TYPE'     )}
sub isnotnull  { return wantarray ? @{shift->fetchinternal('ISNOTNULL')}: shift->fetchinternal('ISNOTNULL')}
sub isprikey   { return wantarray ? @{shift->fetchinternal('ISPRIKEY' )}: shift->fetchinternal('ISPRIKEY' )}
sub isnum     { return wantarray ? @{shift->fetchinternal('ISNUM' )}: shift->fetchinternal('ISNUM' )}
sub isblob     { return wantarray ? @{shift->fetchinternal('ISBLOB' )}: shift->fetchinternal('ISBLOB' )}
sub length     { return wantarray ? @{shift->fetchinternal('LENGTH'   )}: shift->fetchinternal('LENGTH'   )}
sub maxlength { return wantarray ? @{shift->fetchinternal('MAXLENGTH'   )}: shift->fetchinternal('MAXLENGTH'   )}

sub listindices {
    my($sth) = shift;
    my(@result,$i);
    return () unless &Mysql::IDX_TYPE;
    foreach $i (0..$sth->numfields-1) {
    next unless $sth->type->[$i] == &Mysql::IDX_TYPE;
    push @result, $sth->name->[$i];
    }
    @result;
}

sub AUTOLOAD {
    my $meth = $AUTOLOAD;
    $meth =~ s/^.*:://;
    $meth =~ s/_//g;
    $meth = lc($meth);

    # Allow them to say fetch_row or FetchRow
    no strict;
    if (defined &$meth) {
    *$AUTOLOAD = \&{$meth};
    return &$AUTOLOAD(@_);
    }
    Carp::croak ("$AUTOLOAD not defined and not autoloadable");
}

sub unctrl {
    my($x) = @_;
    $x =~ s/\\/\\\\/g;
    $x =~ s/([\001-\037\177])/sprintf("\\%03o",unpack("C",$1))/eg;
    $x;
}

sub optimize {
    my($self,$arg) = @_;
    if (defined $arg) {
    return $self->{'OPTIMIZE'} = $arg;
    } else {
    return $self->{'OPTIMIZE'} ||= $OPTIMIZE;
    }

}

sub as_string {
    my($sth) = @_;
    my($plusline,$titline,$sprintf) = ('+','|','|');
    my($result,$s,$l);
    for (0..$sth->numfields-1)
    {
	$l=length($sth->name->[$_]);
	$l= $sth->maxlength->[$_] if $l < $sth->maxlength->[$_];
	$l=4 if (!$sth->isnotnull->[$_] && $l < 4);
        $plusline .= sprintf "%$ {l}s+", "-" x $l;
        $l= -$l  if (!$sth->isnum->[$_]);
        $titline .= sprintf "%$ {l}s|", $sth->name->[$_];
        $sprintf .= "%$ {l}s|";
    }
    $sprintf .= "\n";
    $result = "$plusline\n$titline\n$plusline\n";
    $sth->dataseek(0);
    my(@row);
    while ((@row = $sth->fetchrow))
    {
      my(@prow);
      foreach (0..$#row)
      {
	$prow[$_] = defined $row[$_] ? unctrl($row[$_]) : "NULL";
      }
      $result .= sprintf $sprintf, @prow;
    }
    $result .= "$plusline\n";
    if ($sth->numrows ne "N/A")
    {
      $s = $sth->numrows == 1 ? "" : "s";
      $result .= $sth->numrows . " row$s processed\n\n";
    }
    return $result;
}

1;
__END__
